﻿using Microsoft.Office.Interop.Outlook;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;

namespace TestProject
{
    /// <summary>
    /// 메일 항목 확장
    /// </summary>
    public static class MailItemExtensions
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// PR_TRANSPORT_MESSAGE_HEADERS
        /// </summary>
        private const string PR_TRANSPORT_MESSAGE_HEADERS = "http://schemas.microsoft.com/mapi/proptag/0x007D001F";

        /// <summary>
        /// 정규식 패턴
        /// </summary>
        private const string REGULAR_EXPRESSION_PATTERN = @"^(?<header_key>[-A-Za-z0-9]+)(?<seperator>:[ \t]*)" +
                                                           "(?<header_value>([^\r\n]|\r\n[ \t]+)*)(?<terminator>\r\n)";

        #endregion
 
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 메일 헤더 구하기 - GetMailHeader(mailItem)

        /// <summary>
        /// 메일 헤더 구하기
        /// </summary>
        /// <param name="mailItem">메일 항목</param>
        /// <returns>메일 헤더</returns>
        public static string GetMailHeader(this MailItem mailItem)
        {
            return (string)mailItem.PropertyAccessor.GetProperty(PR_TRANSPORT_MESSAGE_HEADERS);
        }

        #endregion
        #region 헤더 룩업 구하기 - GetHeaderLookup(mailItem)

        /// <summary>
        /// 헤더 룩업 구하기
        /// </summary>
        /// <param name="mailItem">메일 항목</param>
        /// <returns>헤더 룩업</returns>
        public static ILookup<string, string> GetHeaderLookup(this MailItem mailItem)
        {
            string mailHeader = mailItem.GetMailHeader();

            IEnumerable<Match> matchEnumerable = Regex.Matches(mailHeader, REGULAR_EXPRESSION_PATTERN, RegexOptions.Multiline).Cast<Match>();

            return matchEnumerable.ToLookup(h => h.Groups["header_key"].Value, h => h.Groups["header_value"].Value);
        }

        #endregion
        #region 메일 헤더 값 배열 구하기 - GetMailHeaderValueArray(mailItem, headerName)

        /// <summary>
        /// 메일 헤더 값 배열 구하기
        /// </summary>
        /// <param name="mailItem">메일 항목</param>
        /// <param name="headerName">헤더명</param>
        /// <returns>메일 헤더 값 배열</returns>
        public static string[] GetMailHeaderValueArray(this MailItem mailItem, string headerName)
        {
            ILookup<string, string> headerLookup = mailItem.GetHeaderLookup();

            if(headerLookup.Contains(headerName))
            {
                return headerLookup[headerName].ToArray();
            }


            return new string[0];
        }

        #endregion
    }
}